/*
 *  linux/arch/arm/lib/copypage-feroceon.S
 *
 *  Copyright (C) 1995-1999 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This handles copy_user_page and clear_user_page on Feroceon.  This
 * implementation is mostly copied from the v4wb implementation, but
 * slightly tweaked to deal with write-allocate caches (Feroceon can
 * run L1 D in WA mode), and the 4-word load/store multiple instructions
 * replaced by 8-word variants to maximise memory bandwidth.
 */
#include <linux/linkage.h>
#include <linux/init.h>
#include <asm/asm-offsets.h>

	.text
	.align	5

ENTRY(feroceon_copy_user_page)
	stmfd	sp!, {r4-r8, lr}
	mov	r2, #PAGE_SZ/32
1:	ldmia	r1!, {r3-r8, ip, lr}
	stmia	r0, {r3-r8, ip, lr}
	mcr	p15, 0, r0, c7, c14, 1		@ clean and invalidate D line
	add	r0, r0, #32
	subs	r2, r2, #1
	bne	1b
	mcr	p15, 0, r1, c7, c10, 4		@ drain WB
	ldmfd	sp!, {r4-r8, pc}

	.align	5

ENTRY(feroceon_clear_user_page)
	stmfd	sp!, {r4-r7, lr}
	mov	r1, #PAGE_SZ/32
	mov	r2, #0
	mov	r3, #0
	mov	r4, #0
	mov	r5, #0
	mov	r6, #0
	mov	r7, #0
	mov	ip, #0
	mov	lr, #0
1:	stmia	r0, {r2-r7, ip, lr}
	mcr	p15, 0, r0, c7, c14, 1		@ clean and invalidate D line
	add	r0, r0, #32
	subs	r1, r1, #1
	bne	1b
	mcr	p15, 0, r1, c7, c10, 4		@ drain WB
	ldmfd	sp!, {r4-r7, pc}

	__INITDATA

	.type	feroceon_user_fns, #object
ENTRY(feroceon_user_fns)
	.long	feroceon_clear_user_page
	.long	feroceon_copy_user_page
	.size	feroceon_user_fns, . - feroceon_user_fns
